//////////////////////////////////////////////////////////////////////////
//
// Defines.
//
// Version: 0.2
// Updated: 11.02.2002
// Updates: TGPS class was deleted and
//          added as part of project    (11.02.2002 -Seppo)
//          The code was preserved      (17.1.2002 -Florian)
//
//////////////////////////////////////////////////////////////////////////

unit Defines;

interface

Uses Windows, Graphics, Messages, classes, extctrls, sysutils;

const
  WM_CHILDNOTYFY     = WM_APP + 400;
  WM_MAINNOTIFY      = WM_APP + 401;
  WM_DOWNLOADNOTIFY  = WM_APP + 402;
  WM_NORMAL_CIRCUTE  = WM_APP + 403;
  WM_MOUSE_CIRCUTE   = WM_APP + 404;
  WM_EMERGENCY       = WM_APP + 405;
  LOAD_MAP           = WM_APP + 408;
  NMEAMSG            = WM_APP + 409;
  MAGELLANMSG        = WM_APP + 410;
  GARMINMSG          = WM_APP + 411;
  MAGELLANFINISHED   = WM_APP + 412;
  GARMINFINISHED     = WM_APP + 413;
  GARMINPOS          = WM_APP + 414;
  WM_SETCAPTIONS     = WM_APP + 415;
  WM_IDLE            = WM_APP + 416;
  WM_SETRANGE        = WM_APP + 417;
  STOP_VIEVPORT      = WM_APP + 418;
  WM_ACTIVATE        = WM_APP + 419;
  WM_AUTOREDRAW      = WM_APP + 420;
  WM_REDRAW          = WM_APP + 421;
  WM_COLLECT         = WM_APP + 422;
  WM_SAVE            = WM_APP + 423;
  WM_LOAD            = WM_APP + 424;
  WM_CONVERT         = WM_APP + 425;


{$IFDEF WIN32}
  BadDLLLoad      = 0;
{$ELSE}
  BadDLLLoad      = 32;
{$ENDIF}

type
  PBitmap = ^TBitmap;
  PImage = ^TImage;

  // Tracer
  // TShowCompas = function(AppHandle:THandle):THandle; stdcall;

  ColorTable = ^colorStruct;
  colorStruct = record
    R         : Byte;
    G         : Byte;
    B         : Byte;
  end;

  MASSTRATEGY = record
    MapRelevance                : Integer;
    MPercentage1                : Integer;
    MPercentage2                : Integer;
    MPercentage3                : Integer;
    MPercentage4                : Integer;
    //MapScales                 Not yet implemented SN
  end;

  GPSSTRATEGY = record
    COMPort                     : Integer;
    BaudRate                    : Dword;
    DataBits                    : Byte;
    Parity                      : Char;
    StopBits                    : Byte;
    NMEA                        : String;
  end;

  TNMEASettings = record
    NMEA                        : String;
    NMEA183Tag                  : String;
    NMEA183TagPosition          : Integer;
    LatitudeTag                 : Char;
    LatitudeTagPosition         : Integer;
    LatitudeValuePosition       : Integer;
    LongitudeTag                : Char;
    LongitudeTagPosition        : Integer;
    LongitudeValuePosition      : Integer;
    TimeValuePosition           : Integer;
    ValidTag                    : Char;
    ValidTagPosition            : Integer;
    SpeedValuePosition          : Integer;
    CourseValuePosition         : Integer;
  end;


  // Coordinate transform related
  TProjection = (pKKJ, pYKJ, pUTM);

  TMapDatum = (
    Airy,
    AustralianNational,
    Bessel1841,
    Bessel1841Nambia,
    Clarke1866,
    Clarke1880,
    Everest,
    Fischer1960Mercury,
    Fischer1968,
    GRS1967,
    GRS1980,
    Helmert1906,
    Hough,
    International,
    Krassovsky,
    ModifiedAiry,
    ModifiedEverest,
    ModifiedFischer1960,
    SouthAmerican1969,
    WGS60,
    WGS66,
    WGS72,
    WGS84);

  TEllipsoid = record
    Name                    : string;
    Major, Minor            : Double;
  end;

  TCoordinate = record
    Latitude                : Double;
    Longitude               : Double;
    Datum                   : TMapDatum;
  End;

  TProjected = record
    Northing                : Double;
    Easting                 : Double;
    ZoneNumber              : Integer;
    Datum                   : TMapDatum;
  End;

  EConversionInvalid = class(Exception);

  RelRec = record
    Name                        : String;
    Relevance                   : Real;
  End;

  RelevanceList= array[1..1000] of RelRec;


const
  Ellipsoid: array[TMapDatum] of TEllipsoid  = (
  (Name:'Airy';                  Major:6377563; Minor:6356257),
  (Name:'Australian National';   Major:6378160; Minor:6356775),
  (Name:'Bessel 1841';           Major:6377397; Minor:6356079),
  (Name:'Bessel 1841 (Nambia) '; Major:6377484; Minor:6356166),
  (Name:'Clarke 1866';           Major:6378206; Minor:6356583),
  (Name:'Clarke 1880';           Major:6378249; Minor:6356515),
  (Name:'Everest';               Major:6377276; Minor:6356075),
  (Name:'Fischer 1960(Mercury)'; Major:6378166; Minor:6356784),
  (Name:'Fischer 1968';          Major:6378150; Minor:6356768),
  (Name:'GRS 1967';              Major:6378160; Minor:6356775),
  (Name:'GRS 1980';              Major:6378137; Minor:6356752),
  (Name:'Helmert 1906';          Major:6378200; Minor:6356818),
  (Name:'Hough';                 Major:6378270; Minor:6356794),
  (Name:'International';         Major:6378388; Minor:6356912),
  (Name:'Krassovsky';            Major:6378245; Minor:6356863),
  (Name:'Modified Airy';         Major:6377340; Minor:6356034),
  (Name:'Modified Everest';      Major:6377304; Minor:6356103),
  (Name:'Modified Fischer 1960'; Major:6378155; Minor:6356773),
  (Name:'South American 1969';   Major:6378160; Minor:6356775),
  (Name:'WGS 60';                Major:6378165; Minor:6356783),
  (Name:'WGS 66';                Major:6378145; Minor:6356760),
  (Name:'WGS 72';                Major:6378135; Minor:6356751),
  (Name:'WGS 84';                Major:6378137; Minor:6356752));



function GetProjectionByName(DatumName:String):TProjection;
function GetDatumByName(DatumName:String):TMapDatum;
function ScaleTextToInt(ScaleText: String): Integer;
procedure ReplaceOccurrances(var SourceStr:AnsiString; const TargetStr,NewStr:AnsiString);


implementation


//////////////////////////////////////////////////////////////////////////
// Service functions
//////////////////////////////////////////////////////////////////////////
function GetDatumByName(DatumName:String):TMapDatum;
Begin
  Result := WGS84;
  if (DatumName = 'Airy') Then Result := Airy;
  if (DatumName = 'AustralianNational') Then Result := AustralianNational;
  if (DatumName = 'Bessel1841') Then Result := Bessel1841;
  if (DatumName = 'Bessel1841Nambia') Then Result := Bessel1841Nambia;
  if (DatumName = 'Clarke1866') Then Result := Clarke1866;
  if (DatumName = 'Clarke1880') Then Result := Clarke1880;
  if (DatumName = 'Everest') Then Result := Everest;
  if (DatumName = 'Fischer1960Mercury') Then Result := Fischer1960Mercury;
  if (DatumName = 'Fischer1968') Then Result := Fischer1968;
  if (DatumName = 'GRS1967') Then Result := GRS1967;
  if (DatumName = 'GRS1980') Then Result := GRS1980;
  if (DatumName = 'Helmert1906') Then Result := Helmert1906;
  if (DatumName = 'Hough') Then Result := Hough;
  if (DatumName = 'International') Then Result := International;
  if (DatumName = 'Krassovsky') Then Result := Krassovsky;
  if (DatumName = 'ModifiedAiry') Then Result := ModifiedAiry;
  if (DatumName = 'ModifiedEverest') Then Result := ModifiedEverest;
  if (DatumName = 'ModifiedFischer1960') Then Result := ModifiedFischer1960;
  if (DatumName = 'SouthAmerican1969') Then Result := SouthAmerican1969;
  if (DatumName = 'WGS60') Then Result := WGS60;
  if (DatumName = 'WGS66') Then Result := WGS66;
  if (DatumName = 'WGS72') Then Result := WGS72;
  if (DatumName = 'WGS84') Then Result := WGS84;
End;

function GetProjectionByName(DatumName:String):TProjection;
Begin
  Result := pUTM;
  if (DatumName = 'KKJ') Then Result := pKKJ;
  if (DatumName = 'YKJ') Then Result := pYKJ;
  if (DatumName = 'UTM') Then Result := pUTM;
End;

//////////////////////////////////////////////////////////////////////////
procedure ReplaceOccurrances(var SourceStr:AnsiString; const TargetStr,NewStr:AnsiString);
var
  i: Integer;
begin
  i := AnsiPos(TargetStr, SourceStr);
  while i > 0 do
  begin
    SourceStr := Copy(SourceStr, 1, i-1 ) + NewStr +
                 Copy(SourceStr, i+Length(TargetStr), Length(SourceStr) - i+Length(TargetStr) +1 );
    i := AnsiPos(TargetStr, SourceStr);
  end;
end;


//////////////////////////////////////////////////////////////////////////
function ScaleTextToInt(ScaleText: String): Integer;
var
  i                         : Integer;
  ScaleInt                  : Integer;
begin
  ReplaceOccurrances(ScaleText, ' ','' ); // remove and/or convert exptected substrings
  ReplaceOccurrances(ScaleText, '1:','');
  ReplaceOccurrances(ScaleText, '.','' );
  ReplaceOccurrances(ScaleText, 'milj','000000');
  ReplaceOccurrances(ScaleText, 'mill','000000');

  for i:=1 to Length(ScaleText) do        // remove all but numbers
    if (ScaleText[i]<'0') or (ScaleText[i]>'9') then delete(ScaleText, i, 1);

  val(ScaleText, ScaleInt, i);            // convert to Int

  if i<>0 then Result := -1
  else Result := ScaleInt;
end;



end.

