//////////////////////////////////////////////////////////////////////////
//
// A map
//
// Version: 0.1
// Updated: 17.01.2002
// Updates: The code was preserved (17.1.2002 -Florian)
//
//////////////////////////////////////////////////////////////////////////

unit MapObj;

interface

Uses Defines, UnitLocation;

type
  TMap = class
  public
    Image               : PImage;
    MapTitle            : String;
    MapID               : Integer;
    ImageLocation       : ANSIString;
    PictureWidth        : Integer;  // Picture coordinates
    PictureHeight       : Integer;
    XOffset             : Integer;
    YOffset             : Integer;
    LeftBottomX         : Double;   // Real Coordinates
    LeftBottomY         : Double;   // of map boundaries
    RightTopX           : Double;   //
    RightTopY           : Double;   //
    Scale               : Double;

    Loaded              : Boolean;
    Rotate              : Double;   // rotate scale

    constructor Create;
    destructor Destroy; override;
    procedure SetFields(m:TMap);
    procedure SetImage(m:TMap);

    procedure Real2Screen(Coordinate:TCoordinate; var SX, SY:Integer);
    procedure Screen2Real(dX, dY:Integer; var Coordinate:TCoordinate);
    function  WithIn(Coordinate:TCoordinate):Boolean;
  End;

implementation

Uses UnitBUS;

//////////////////////////////////////////////////////////////////////////
// Constructor & Destructor
//////////////////////////////////////////////////////////////////////////
Constructor TMap.Create;
Begin
  Image         :=nil;
  MapID         :=0;
  MapTitle      :='';
  ImageLocation :='';
  PictureWidth  :=0;
  PictureHeight :=0;
  LeftBottomX   :=0;
  LeftBottomY   :=0;
  RightTopX     :=0;
  RightTopY     :=0;
  Loaded        :=False;

  Rotate        :=0;
End;

{-------------------------------------------------}
Destructor TMap.Destroy;
Begin
  MapTitle      := '';
  ImageLocation := '';
  Loaded        :=False;
End;

{-------------------------------------------------}
procedure TMap.SetFields(m:TMap);
Begin
  MapID         := m.MapID;
  MapTitle      := m.MapTitle;
  ImageLocation := m.ImageLocation;
  PictureWidth  := m.PictureWidth;
  PictureHeight := m.PictureHeight;
  LeftBottomX   := m.LeftBottomX;
  LeftBottomY   := m.LeftBottomY;
  RightTopX     := m.RightTopX;
  RightTopY     := m.RightTopY;
  Rotate        := m.Rotate;
  Scale         := m.Scale;
End;

{-------------------------------------------------}
procedure TMap.SetImage(m:TMap);
Begin
  Image.Picture.Bitmap.Assign(m.Image.Picture.Bitmap);
End;

{-------------------------------------------------}
procedure TMap.Real2Screen(Coordinate:TCoordinate; var SX, SY:Integer);
Var
  px, py                    : Double;
  tx, ty                    : Double;
  lbx, lby, rty             : Double;
  Location                  : TLocation;
  Projected                 : TProjected;
  t_coord                   : TCoordinate;
  Datum                     : TMapDatum;
  Projection                : TProjection;
  sinR                      : Double;   //
  cosR                      : Double;   //
Begin
  Projection := GetProjectionByName(BUS.DefProjection);
  Datum      := GetDatumByName(BUS.DefDatum);

  SX := 0; SY :=0;
  Location := TLocation.Create;

  Coordinate.Datum := Datum;
  Location.LatLon2Projected(Projection, Coordinate, Projected);
  tx := Projected.Easting;
  ty := Projected.Northing;

  t_coord.Latitude := LeftBottomY;
  t_coord.Longitude:= LeftBottomX;
  t_coord.Datum    := Datum;
  Location.LatLon2Projected(Projection, t_coord, Projected);
  lbx := Projected.Easting;
  lby := Projected.Northing;

  t_coord.Latitude := RightTopY;
  t_coord.Longitude:= RightTopX;
  t_coord.Datum    := Datum;
  Location.LatLon2Projected(Projection, t_coord, Projected);
  rty := Projected.Northing;

  Scale := (rty-lby)/pictureHeight;

  px := (tx - lbx)/Scale;
  py := (ty - lby)/Scale;

  CosR := cos(Rotate/180*pi);
  SinR := sin(Rotate/180*pi);
  SX := round(px*cosR + py*sinR);
  SY := pictureHeight - round(py*cosR - px*sinR);

  dec(SX,XOffset);
  dec(SY,YOffset);

  Location.Free;
End;

{-------------------------------------------------}
procedure TMap.Screen2Real(dX, dY:Integer; var Coordinate:TCoordinate);
Var
  gx,gy                     : Double;
  Location                  : TLocation;
  Projected, t_Proj         : TProjected;
  t_coord                   : TCoordinate;
  Datum                     : TMapDatum;
  Projection                : TProjection;
  sinR                      : Double;   //
  cosR                      : Double;   //

Begin
  inc(dX,XOffset);
  inc(dY,YOffset);

  Projection := GetProjectionByName(BUS.DefProjection);
  Datum      := GetDatumByName(BUS.DefDatum);

  Location := TLocation.Create;
  t_coord.Latitude := LeftBottomY;
  t_coord.Longitude:= LeftBottomX;
  t_coord.Datum    := Datum;
  Location.LatLon2Projected(Projection, t_coord, Projected);

  if (Scale=0) Then
  Begin
    t_coord.Latitude := RightTopY;
    t_coord.Longitude:= RightTopX;
    t_coord.Datum    := Datum;
    Location.LatLon2Projected(Projection, t_coord, t_Proj);

    Scale := (t_Proj.Northing-Projected.Northing)/pictureHeight;
  End;

  gx:=dX*Scale;
  gy:=(pictureHeight-dY-1)*Scale;

  CosR := cos(Rotate/180*pi);
  SinR := sin(Rotate/180*pi);
  Projected.Easting  := Projected.Easting+gx*cosR-gy*sinR;
  Projected.Northing := Projected.Northing+gy*cosR+gx*sinR;

  Location.Projected2LatLon(Projection, Projected, Coordinate);

  Location.Free;
End;

{-------------------------------------------------}
function TMap.WithIn(Coordinate:TCoordinate):Boolean;
Begin
  Result := False;
  if (Coordinate.Longitude>LeftBottomX) and (Coordinate.Longitude<RightTopX) and
     (Coordinate.Latitude> LeftBottomY) and (Coordinate.Latitude<RightTopY) Then
  Result := True;
End;



end.
