//////////////////////////////////////////////////////////////////////////
//
// Image format stuff.
//
// Version: 0.5
// Updates: 05.02.2002 Pavel Kopylov (Saving abilities)
//          25.01.2002 Pavel Kopylov
//          06.02.2002 Pavel Kopylov => Unnecessary functionality is moved to MISS
//          The code was preserved (17.1.2002 -Florian)
//
//////////////////////////////////////////////////////////////////////////

unit Pictures;

interface

uses Windows, SysUtils, Dialogs, graphics, extctrls, Math,
     Defines;

function  GIF_to_RAW(InputName:String; var Data:Pointer; var XSize,YSize:LongWord; QL:Integer):Integer;cdecl; external 'Converter.dll';
function  PNG_to_RAW(InputName:String; var Data:Pointer; var XSize,YSize:LongWord; QL:Integer):Integer;cdecl; external 'Converter.dll';
function  PPM_to_RAW(InputName:String; var Data:Pointer; var XSize,YSize:LongWord; QL:Integer):Integer;cdecl; external 'Converter.dll';
function  MISS_to_RAW(InputName:String; var Data:Pointer; var XSize,YSize:LongWord; QL:Integer):Integer;cdecl; external 'Converter.dll';
function  RAW_to_GIF(OutputName:String; Data:Pointer; XSize,YSize:LongWord; QL:Integer):Integer;cdecl; external 'Converter.dll';
function  RAW_to_PNG(OutputName:String; Data:Pointer; XSize,YSize:LongWord; QL:Integer):Integer;cdecl; external 'Converter.dll';
function  RAW_to_PPM(OutputName:String; Data:Pointer; XSize,YSize:LongWord; QL:Integer):Integer;cdecl; external 'Converter.dll';
function  RAW_to_MISS(OutputName:String; Data:Pointer; XSize,YSize,ClusterX,ClusterY,Colors:LongWord;
                      LeftBottomX,LeftBottomY,RightTopX,RigthTopY,Scale,Rotate:Double; QL:Integer):Integer;cdecl; external 'Converter.dll';

procedure DeallocateData(Data:Pointer);cdecl; external 'Converter.dll';

type
  TPictures = class
  public
    XSize, YSize                 : DWORD;

    constructor Create;
    destructor Destroy; override;
    function LoadImg(Name:String; var map:PImage):Integer;
    function SaveImg(Name:String; map:PImage; CX,CY,CO:LongWord; LBX,LBY,RTX,RTY,SC,RT:Double):Integer;
  end;

implementation

//////////////////////////////////////////////////////////////////////////
// Constructor & Destructor
//////////////////////////////////////////////////////////////////////////
Constructor TPictures.Create;
Begin
  {}
End;

Destructor TPictures.Destroy;
Begin
  {}
End;


//////////////////////////////////////////////////////////////////////////
// Load to memory
//////////////////////////////////////////////////////////////////////////
function TPictures.LoadImg(Name:String; var map:PImage):Integer;
Var
  ExtStr                         : String;
  P                              : PByteArray;
  x, y                           : LongWord;
  Data                           : Pointer;
  tmp                            : PChar;
Begin
  Data := nil;
  Result  := 0;

  ExtStr := AnsiLowerCase(ExtractFileExt(Name));
  if (extStr = '.bmp') Then
  Begin
    map.picture.loadfromfile(Name);
    exit;
  End
  else
    if (extStr = '.pbm') or (extStr = '.pgm') or (extStr = '.ppm')
    Then Result := PPM_to_RAW(Name, Data, XSize, YSize, 0)
    else
      if (extStr = '.gif') Then Result := GIF_to_RAW(Name, Data, XSize, YSize, 0)
      else
        if (extStr = '.png') Then Result := PNG_to_RAW(Name, Data, XSize, YSize, 0)
        else
          if (extStr = '.mis') Then Result := MISS_to_RAW(Name, Data, XSize, YSize, 0);

  if (Result=0) Then Exit;

  map^.Picture.BitMap := TBitMap.create;
  map.Picture.BitMap.Width       := xSize;
  map.Picture.BitMap.Height      := ySize;
  map.Picture.BitMap.PixelFormat := pf24bit; // GIF have MAX 256 colors
  map.Width  := xSize;
  map.Height := ySize;

  tmp := Data;
  for y := 0 to ySize-1 do
  begin
    P := map.Picture.BitMap.ScanLine[y];
    for x := 0 to xSize-1 do
    begin
      P[x*3+2] := ord(tmp^);  inc(tmp);            // BGR
      P[x*3+1] := ord(tmp^);  inc(tmp);
      P[x*3]   := ord(tmp^);  inc(tmp);
    end
  end;
  DeallocateData(Data);
End;



//////////////////////////////////////////////////////////////////////////
// Reconstruct from Image
//////////////////////////////////////////////////////////////////////////
function TPictures.SaveImg(Name:String; map:PImage; CX,CY,CO:LongWord; LBX,LBY,RTX,RTY,SC,RT:Double):Integer;
Var
  ExtStr                         : String;
  P                              : PByteArray;
  x, y                           : LongWord;
  Data                           : Pointer;
  tmp                            : PChar;
Begin
  getmem(Data, map.Width*map.Height*3);
  tmp   := Data;
  XSize := map.Width;
  YSize := map.Height;
  Result:= 0;

  for y := 0 to YSize-1 do
  begin
    P := map.Picture.BitMap.ScanLine[y];
    for x := 0 to XSize-1 do
    begin
      tmp^ := chr(P[x*3+2]);  inc(tmp);
      tmp^ := chr(P[x*3+1]);  inc(tmp);
      tmp^ := chr(P[x*3]);    inc(tmp);
    end
  end;

  ExtStr := AnsiLowerCase(ExtractFileExt(Name));
  if (extStr = '.bmp') Then
  Begin
    map.picture.savetofile(Name);
    exit;
  End
  else
    if (extStr = '.pbm') or (extStr = '.pgm') or (extStr = '.ppm')
    Then Result := RAW_to_PPM(Name, Data, XSize, YSize, 0)
    else
      if (extStr = '.gif') Then Result := RAW_to_GIF(Name, Data, XSize, YSize, 0)
      else
        if (extStr = '.png') Then Result := RAW_to_PNG(Name, Data, XSize, YSize, 0)
        else
          if (extStr = '.mis') Then Result := RAW_to_MISS(Name,Data,XSize,YSize,CX,CY,CO,LBX,LBY,RTX,RTY,SC,RT,0);
  FreeMem(Data);
End;


end.
