//////////////////////////////////////////////////////////////////////////
//
// Conversions between Planar (projected) and ... (LatLon) coordinate systems
//
// Version: 0.1
// Updated: 17.01.2002
// Updates: The code was preserved (17.1.2002 -Florian)
//
//////////////////////////////////////////////////////////////////////////

unit UnitCoordConv;

interface

uses SysUtils, Forms,
     Defines, UnitKKJ, UnitYKJ, UnitUTM;


type
  TCoordConvert = class(TObject)
  private
    UTM                     : TUTM;
    KKJ                     : TKKJ;
    YKJ                     : TYKJ;
  public

    Constructor Create;
    Destructor  Destroy; override;

    procedure   LatLon2Projected(Projection:TProjection; Coordinate:TCoordinate; Var Projected : TProjected);
    procedure   Projected2LatLon(Projection:TProjection; Projected : TProjected; Var Coordinate:TCoordinate);
  end;


implementation

//////////////////////////////////////////////////////////////////////////
// Constuctor - Destructor
//////////////////////////////////////////////////////////////////////////
constructor TCoordConvert.Create;
begin
  UTM := TUTM.Create;
  KKJ := TKKJ.Create;
  YKJ := TYKJ.Create;
end;


Destructor TCoordConvert.Destroy;
begin
  UTM.free;
  KKJ.free;
  YKJ.free;
end;

//////////////////////////////////////////////////////////////////////////
// Interface
//////////////////////////////////////////////////////////////////////////
procedure TCoordConvert.LatLon2Projected(Projection:TProjection; Coordinate:TCoordinate; Var Projected : TProjected);
Begin
  case Projection of
    pKKJ: KKJ.ForwardTransform(Coordinate, Projected);
    pYKJ: YKJ.ForwardTransform(Coordinate, Projected);
    pUTM: UTM.ForwardTransform(Coordinate, Projected);
  End;
End;


procedure TCoordConvert.Projected2LatLon(Projection:TProjection; Projected : TProjected; Var Coordinate:TCoordinate);
Begin
  case Projection of
    pKKJ: KKJ.InverseTransform(Projected,Coordinate);
    pYKJ: YKJ.InverseTransform(Projected,Coordinate);
    pUTM: UTM.InverseTransform(Projected,Coordinate);
  End;
End;


end.
