//////////////////////////////////////////////////////////////////////////
//
// Latitude ans Longitude in ggmm.sss format.
//
// Version: 0.1
// Updated: 17.01.2002
// Updates: The code was preserved (17.1.2002 -Florian)
//
//////////////////////////////////////////////////////////////////////////

unit UnitLocation;

interface

uses SysUtils,
     UnitCoordConv, Defines;

type
  TLocation = class(TCoordConvert)
  private
    FDateTime       : TDateTime;
    FSpeed          : Double;
    FCourse         : Double;
    Coordinate      : TCoordinate;
  public
    Constructor Create;
    Destructor Destroy; override;

    procedure   SetCourse(NewCourse : Double);
    function    GetTime : TDateTime;
    procedure   SetTime(NewTime : TDateTime);
    function    GetCourse : Double;
    procedure   SetSpeed(NewSpeed : Double);
    function    GetSpeed : Double;
    procedure   SetCoordinate(NewCoordinate:TCoordinate);
    function    GetCoordinate : TCoordinate;
    function    GetCorners(cCenter:TCoordinate; Scale:Double;
                           Width, Height : Integer;
                           Var cLB, cRT:TCoordinate; Var Rotate:Double):Boolean;
  end;

implementation

Uses UnitBUS;

//////////////////////////////////////////////////////////////////////////
// Constructor & Destructor
//////////////////////////////////////////////////////////////////////////
constructor TLocation.Create;
begin
  inherited Create;

  FDatetime := Now();
end;

{-------------------------------------------------}
Destructor TLocation.Destroy;
begin
  FDateTime :=0;
  FSpeed    :=0;
  FCourse   :=0;

  inherited Destroy;
end;

//////////////////////////////////////////////////////////////////////////
// Interface
//////////////////////////////////////////////////////////////////////////
function TLocation.GetTime : TDateTime;
Begin
  Result := FDateTime;
End;

procedure TLocation.SetTime(NewTime : TDateTime);
Begin
  FDateTime := NewTime;
End;

{-------------------------------------------------}
procedure TLocation.SetCourse(NewCourse : Double);
Begin
  FCourse := NewCourse;
End;

function TLocation.GetCourse : Double;
Begin
  Result := FCourse;
End;

{-------------------------------------------------}
procedure TLocation.SetSpeed(NewSpeed : Double);
Begin
  FSpeed := NewSpeed;
End;

function TLocation.GetSpeed : Double;
Begin
  Result := FSpeed;
End;

{-------------------------------------------------}
procedure TLocation.SetCoordinate(NewCoordinate:TCoordinate);
Begin
  Coordinate.Latitude  := NewCoordinate.Latitude;
  Coordinate.Longitude := NewCoordinate.Longitude;
  Coordinate.Datum     := NewCoordinate.Datum;
End;

function TLocation.GetCoordinate:TCoordinate;
Begin
  Result.Latitude  := Coordinate.Latitude;
  Result.Longitude := Coordinate.Longitude;
  Result.Datum     := Coordinate.Datum;
End;
{-------------------------------------------------}

function TLocation.GetCorners(cCenter:TCoordinate; Scale:Double; Width, Height : Integer;
                              Var cLB, cRT:TCoordinate; Var Rotate:Double):Boolean;
Var
  cLT                       : TCoordinate;
  pCenter, pLB, pLT, pRT    : TProjected;
  Datum                     : TMapDatum;
  Projection                : TProjection;
Begin
  Projection := GetProjectionByName(BUS.DefProjection);
  Datum      := GetDatumByName(BUS.DefDatum);

  cCenter.Datum := Datum;

  LatLon2Projected(Projection, cCenter, pCenter);

  pLB.Northing := pCenter.Northing - Scale/2*Height;
  pLB.Easting  := pCenter.Easting - Scale/2*Width;
  Projected2LatLon(Projection, pLB, cLB);

  pRT.Northing := pCenter.Northing + Scale/2*Height;
  pRT.Easting  := pCenter.Easting + Scale/2*Width;
  Projected2LatLon(Projection, pRT, cRT);


  if (Scale<25) Then                     // Rotate angle calculation
  Begin                                  //
    // Now back to meters                // The trick is to use diff. projections
    LatLon2Projected(pKKJ, cLB, pLB);    //  "--> (YKJ) -->" and "<-- (KKJ) <--"
    LatLon2Projected(pKKJ, cRT, pRT);    //
                                         // Note LT - left top
    pLT.Datum    := Datum;               //      LB - left bottom
    pLT.Northing := pRT.Northing;        //      RT - right top
    pLT.Easting  := pLB.Easting;
    pLT.ZoneNumber := pLB.ZoneNumber;

    // Back to coordinates
    Projected2LatLon(pKKJ, pLB, cLB);
    Projected2LatLon(pKKJ, pLT, cLT);

    // And finally back to meters
    LatLon2Projected(pYKJ, cLB, pLB);
    LatLon2Projected(pYKJ, cLT, pLT);

    // Now dhe rotate...
    Rotate := ArcTan(abs(pLT.Easting-pLB.Easting)/abs(pLT.Northing-pLB.Northing))*180/pi;
  End;

  result := True;
End;

end.
