//////////////////////////////////////////////////////////////////////////
//
// MISS
//
// Version: 0.2
// Created: 06.02.2002 Pavel Kopylov
//
//////////////////////////////////////////////////////////////////////////

unit UnitMISS;

interface

Uses Windows, Dialogs, Graphics, SysUtils,
     Defines, MapObj, UnitLocation;

//MISS
function OpenMISSFile(Name:String):pointer;cdecl; external 'MISS_DLL.dll';
function MISSPages(ptr:pointer):DWORD;cdecl; external 'MISS_DLL.dll';
function OpenMISSPage(prt:pointer; NPage:DWORD):Integer;cdecl; external 'MISS_DLL.dll';
function MISSPageLayers(prt:pointer):DWORD;cdecl; external 'MISS_DLL.dll';
function MISSPageWidth(prt:pointer):DWORD;cdecl; external 'MISS_DLL.dll';
function MISSPageHeight(prt:pointer):DWORD;cdecl; external 'MISS_DLL.dll';
function MISSPageBackground(prt:pointer):DWORD;cdecl; external 'MISS_DLL.dll';
function MISSLayerWidth(prt:pointer; NLayer:DWORD):DWORD;cdecl; external 'MISS_DLL.dll';
function MISSLayerHeight(prt:pointer; NLayer:DWORD):DWORD;cdecl; external 'MISS_DLL.dll';
function MISSLayerClusterWidth(prt:pointer; NLayer:DWORD):DWORD;cdecl; external 'MISS_DLL.dll';
function MISSLayerClusterHeight(prt:pointer; NLayer:DWORD):DWORD;cdecl; external 'MISS_DLL.dll';
function MISSLayerColor(prt:pointer; NLayer:DWORD):DWORD;cdecl; external 'MISS_DLL.dll';
function MISSDecodeLayer(prt:pointer; NLayer:DWORD; Target:pointer; var IsNotEmpty:Integer):pointer;cdecl; external 'MISS_DLL.dll';
function MISSDecodeCluster(prt:pointer; NLayer:DWORD; NCluster:DWORD; Target:pointer; var IsNotEmpty:Integer):pointer;cdecl; external 'MISS_DLL.dll';
function MISSDecodeLayerEx(prt:pointer; NLayer:DWORD; Target:pointer; var IsNotEmpty:Integer):pointer;cdecl; external 'MISS_DLL.dll';
function MISSDecodeClusterEx(prt:pointer; NLayer:DWORD; NCluster:DWORD; Target:pointer; var IsNotEmpty:Integer):pointer;cdecl; external 'MISS_DLL.dll';
function MISSDecodeClusterA(prt:pointer; NCluster:DWORD; Target:pointer; var IsNotEmpty:Integer):pointer;cdecl; external 'MISS_DLL.dll';
function MISSPageLBX(prt:pointer):double;cdecl; external 'MISS_DLL.dll';
function MISSPageLBY(prt:pointer):double;cdecl; external 'MISS_DLL.dll';
function MISSPageRTX(prt:pointer):double;cdecl; external 'MISS_DLL.dll';
function MISSPageRTY(prt:pointer):double;cdecl; external 'MISS_DLL.dll';
function MISSPageScale(prt:pointer):double;cdecl; external 'MISS_DLL.dll';
function MISSPageRotate(prt:pointer):double;cdecl; external 'MISS_DLL.dll';
procedure CloseMISSPage(prt:pointer);cdecl; external 'MISS_DLL.dll';
procedure CloseMISSFile(prt:pointer);cdecl; external 'MISS_DLL.dll';



type
  TMISSImage = class
  public
    XSize                        : DWORD;
    YSize                        : DWORD;
    ClusterWidth                 : DWORD;
    ClusterHeight                : DWORD;
    ClustersX                    : DWORD;
    ClustersY                    : DWORD;

    constructor Create;
    destructor Destroy; override;

    function  Initialize(MapName:String; NPage,NLayer:DWORD):Boolean;
    procedure GetMISSGeoInfo(Var LBX,LBY,RTX,RTY,Rotate,Scale:Double);
    procedure GetMISSCluster(NLayer, NCluster: DWORD; var bmp:PBitmap);
    procedure GetMISSClusterA(NCluster: DWORD; var bmp:PBitmap);
    function  ClusterAtXY(X,Y:DWORD):DWORD;
    procedure ClusterPos(NCluster:DWORD; Var X,Y:DWORD);

    procedure LoadMapInfo(var Map:TMap);
    procedure SetRequest(CurrentLocation:TLocation; Scale:LongWord);

  Private
    m_MapName                    : String;
    m_miss                       : Pointer;
    m_CurrentPage                : DWORD;
    m_CurrentLayer               : DWORD;

    m_LBX                        : Double;
    m_LBY                        : Double;
    m_RTX                        : Double;
    m_RTY                        : Double;
    m_Scale                      : DWORD;
    m_Rotate                     : Double;

    m_RequestLat                 : Double;
    m_RequestLon                 : Double;
  end;

implementation


//////////////////////////////////////////////////////////////////////////
// Constructor & Destructor
//////////////////////////////////////////////////////////////////////////
Constructor TMISSImage.Create;
Begin
  m_miss :=nil;
End;


///////////////////////////////////////////////////////////////
Destructor TMISSImage.Destroy;
Begin
  if (m_miss <> nil) Then
  Begin
    CloseMISSPage(m_miss);
    CloseMISSFile(m_miss);
  end;
End;


///////////////////////////////////////////////////////////////
// MISS interface
///////////////////////////////////////////////////////////////
procedure TMISSImage.SetRequest(CurrentLocation:TLocation; Scale:LongWord);
Var
  t_Coordinate               : TCoordinate;

Begin
  t_Coordinate := CurrentLocation.GetCoordinate;
  m_RequestLat := t_Coordinate.Latitude;
  m_RequestLon := t_Coordinate.Longitude;
  m_Scale      := Scale;
End;


function TMISSImage.Initialize(MapName:String; NPage,NLayer:DWORD):Boolean;
Var
  ret                        : Integer;
  NPages, NLayers            : DWORD;
Begin
  Result := False;

  m_MapName := MapName;
  m_CurrentPage := NPage;
  m_CurrentLayer := NLayer;

  m_miss := OpenMISSFile(MapName);               // Open MISS file
  if m_miss = nil Then
  begin
    MessageDlg('Error: reading MISS file', mtError, [mbOk], 0);
    exit;
  end;
  NPages := MISSPages(m_miss);

  // Pages //////////////////////////////////////////////////////////
  if (m_CurrentPage>NPages) Then
  Begin
    MessageDlg('Error: reading MISS page', mtError, [mbOk], 0);
    CloseMISSFile(m_miss);
    m_miss := nil;
    exit;
  End;

  ret := OpenMISSPage(m_miss, m_CurrentPage);    // Open Page
  if ret = 0 Then
  begin
    MessageDlg('Error: reading MISS page', mtError, [mbOk], 0);
    CloseMISSFile(m_miss);
    m_miss := nil;
    exit;
  end;

  m_LBX   := MISSPageLBX(m_miss);
  m_LBY   := MISSPageLBY(m_miss);
  m_RTX   := MISSPageRTX(m_miss);
  m_RTY   := MISSPageRTY(m_miss);
  m_Rotate:= MISSPageRotate(m_miss);
  m_Scale := round(MISSPageScale(m_miss));
  XSize   := MISSPageWidth(m_miss);
  YSize   := MISSPageHeight(m_miss);


  // Layers /////////////////////////////////////////////////////////
  NLayers := MISSPageLayers(m_miss);
  if (m_CurrentLayer>NLayers) Then
  Begin
    MessageDlg('Error: reading MISS layer', mtError, [mbOk], 0);
    CloseMISSPage(m_miss);
    CloseMISSFile(m_miss);
    m_miss := nil;
    exit;
  End;

  ClusterWidth := MISSLayerClusterWidth(m_miss, m_CurrentLayer);
  ClusterHeight := MISSLayerClusterHeight(m_miss, m_CurrentLayer);

  if (ClusterWidth <>0) and (ClusterWidth <>XSize)
  then ClustersX := (XSize-1) div ClusterWidth +1
  else ClustersX := 1;

  if (ClusterHeight <>0) and (ClusterHeight <>XSize)
  Then ClustersY := YSize div ClusterHeight
  else ClustersY := 1;

  Result := True;
End;


///////////////////////////////////////////////////////////////
procedure TMISSImage.GetMISSGeoInfo(Var LBX,LBY,RTX,RTY,Rotate,Scale:Double);
Begin
  LBX   := m_LBX;
  LBY   := m_LBY;
  RTX   := m_RTX;
  RTY   := m_RTY;
  Scale := m_Scale;
  Rotate:= m_Rotate;
End;


///////////////////////////////////////////////////////////////
procedure TMISSImage.GetMISSCluster(NLayer, NCluster: DWORD; var bmp:PBitmap);
Var
  x, y                          : DWORD;
  P                             : PByteArray;
  bs                            : Array of byte;
  i, StreamLen                  : DWORD;
  IsNotEmpty                    : Integer;

Begin
  if m_miss=nil then
  Begin
    MessageDlg('Error: file is not loaded', mtError, [mbOk], 0);
    exit;
  end;

  bmp.Width       := ClusterWidth;
  bmp.Height      := ClusterHeight;
  bmp.PixelFormat := pf24bit;

  streamLen := ClusterWidth*ClusterHeight*3;
  setlength(bs, streamLen);

  if (NLayer=0) Then
       bs := MISSDecodeClusterEx(m_miss, NLayer, NCluster, bs, IsNotEmpty)
  Else bs := MISSDecodeCluster(m_miss, NLayer, NCluster, bs, IsNotEmpty);

  if (IsNotEmpty=1) Then
  Begin
    x:=0;
    y:=0;
    P:=bmp.ScanLine[y];

    i:=0;
    while (i<streamLen) and (y<ClusterHeight) do
    Begin
      if (bs[i]<>0) or (bs[i+1]<>0) or (bs[i+2]<>0) Then
      Begin
        P[x*3]  :=bs[i+2];
        P[x*3+1]:=bs[i+1];
        P[x*3+2]:=bs[i];
      end;

      inc(i,3);
      inc(x);
      if (x=ClusterWidth) Then
      Begin
        x:=0;
        inc(y);
        if (y<ClusterHeight) Then P := bmp.ScanLine[y];
      End;
    End;
  End;

  SetLength(bs,0);
End;


///////////////////////////////////////////////////////////////
procedure TMISSImage.GetMISSClusterA(NCluster: DWORD; var bmp:PBitmap);
Var
  X, Y                          : DWORD;
  P                             : PByteArray;
  bs                            : Array of byte;
  i, StreamLen                  : DWORD;
  IsNotEmpty                    : Integer;

Begin
  if m_miss=nil then
  Begin
    MessageDlg('Error: file is not loaded', mtError, [mbOk], 0);
    exit;
  end;

  bmp.Width       := ClusterWidth;
  bmp.Height      := ClusterHeight;
  bmp.PixelFormat := pf24bit;

  streamLen := ClusterWidth*ClusterHeight*3;
  setlength(bs, streamLen);

  bs := MISSDecodeClusterA(m_miss, NCluster, bs, IsNotEmpty);

  if (IsNotEmpty=1) Then
  Begin
    x:=0;
    y:=0;
    P:=bmp.ScanLine[y];

    i:=0;
    while (i<streamLen) and (y<ClusterHeight) do
    Begin
      if (bs[i]<>0) or (bs[i+1]<>0) or (bs[i+2]<>0) Then
      Begin
        P[x*3]  :=bs[i+2];
        P[x*3+1]:=bs[i+1];
        P[x*3+2]:=bs[i];
      end;

      inc(i,3);
      inc(x);
      if (x=ClusterWidth) Then
      Begin
        x:=0;
        inc(y);
        if (y<ClusterHeight) Then P := bmp.ScanLine[y];
      End;
    End;
  End;

  SetLength(bs,0);
End;


///////////////////////////////////////////////////////////////
function TMISSImage.ClusterAtXY(X,Y:DWORD):DWORD;
Begin
  result := (Y div ClusterHeight)*ClustersX + (X div ClusterWidth);
End;


///////////////////////////////////////////////////////////////
procedure TMISSImage.ClusterPos(NCluster:DWORD; Var X,Y:DWORD);
Var
  XOffset, YOffset          : DWORD;
Begin
  XOffset := NCluster mod ClustersX;
  YOffset := NCluster div ClustersX;

  X := ClusterWidth*XOffset;
  Y := ClusterHeight*YOffset;
End;


/////////////////////////////////////////////////////////////////////////
// Loading: the record from file into memory
/////////////////////////////////////////////////////////////////////////
procedure TMISSImage.LoadMapInfo(var Map:TMap);
Var
  NameStr                   : String;

Begin
  NameStr := m_MapName;
  NameStr := ExtractFileName(NameStr);
  setLength(NameStr, Pos('.', NameStr)-1);

  Map.MapTitle      := NameStr;
  Map.ImageLocation := m_MapName;
  Map.MapID         := 0;
  Map.LeftBottomX   := m_LBX;
  Map.LeftBottomY   := m_LBY;
  Map.RightTopX     := m_RTX;
  Map.RightTopY     := m_RTY;
  Map.Rotate        := m_Rotate;
  Map.Scale         := m_Scale;
  Map.XOffset       := 0;
  Map.YOffset       := 0;
  Map.PictureWidth  := XSize;
  Map.PictureHeight := YSize;
End;



end.