//////////////////////////////////////////////////////////////////////////
//
// Universal Transverse Mercator Projection class
// extends   TransverseMercator class
//
// Version: 0.1
// Updated: 17.01.2002
// Updates: The code was preserved (17.1.2002 -Florian)
//
//////////////////////////////////////////////////////////////////////////

// OLD HEADER !
// Universal Transverse Mercator Projection class
// extends   TransverseMercator class
//
// @author     Alexander Kolesnikov
// @created    8. marraskuuta 1999
//   Scale factor: 0.996
//   Datum: any
// Pascal coding   Pavel Kopylov (18.04.2001)

Unit UnitUTM;

Interface

Uses
  UnitMercator, Defines;

type
  TUTM = class(TMercator)
  public
    scale_fact              : Double;         // scale factor

    Constructor Create;
    Destructor  Destroy; override;

    procedure ForwardTransform(Coordinate:TCoordinate; Var Projected:TProjected);
    procedure InverseTransform(Projected:TProjected; Var Coordinate:TCoordinate);
    function  CalcZoneNum(Coordinate:TCoordinate):Integer;
  End;


Implementation

//////////////////////////////////////////////////////////////////////////
// Constructor & Destructor
//////////////////////////////////////////////////////////////////////////
constructor TUTM.Create;
begin
  inherited Create;
  scale_fact := 0.9996;
end;


Destructor TUTM.Destroy;
begin
  inherited Destroy;
end;


//////////////////////////////////////////////////////////////////////////
// Latitude, Longitude -> meters (Northing/Easting)
//////////////////////////////////////////////////////////////////////////
procedure TUTM.ForwardTransform(Coordinate:TCoordinate; Var Projected:TProjected);
Var
  lat, lon                  : Double;
  northVal, eastVal         : Double;
  lon_origin                : Double;
  north_false, east_false   : Double;
  tmpCoord                  : TCoordinate;
  proj_tmp                  : TProjected;
  intZoneNum                : Integer;        // Zone: 1..60;

Begin
  lat   := Coordinate.Latitude;
  lon   := Coordinate.Longitude;

  north_false := 0;      // False Northing;
  east_false  := 500000; // False Easting

  if (lat < -80) or (lat > 84) Then
  Begin
    raise EConversionInvalid.Create('UTM: Imposible value of latitude');
    exit;
  End;

  //Make sure the longitude is between -180.00 .. 179.9
  lon := (lon+180) - trunc((lon+180)/360)*360-180;

  tmpCoord.Latitude  := lat;
  tmpCoord.Longitude := lon;
  tmpCoord.Datum     := Coordinate.Datum;
  intZoneNum  := calcZoneNum(tmpCoord);

  // Set origin of the Zone;
  lon_origin  := intZoneNum*6 - 183;

  if(lat < 0) Then north_false := 10000000;   //10000000 meter offset for southern hemisphere

  try
    forwardConversion(tmpCoord, lon_origin, proj_tmp);
    eastVal  := east_false  + scale_fact * proj_tmp.Easting;
    northVal := north_false + scale_fact * proj_tmp.Northing;
  except
    on EConversionInvalid do exit;
  End;

  Projected.Northing   := northVal;
  Projected.Easting    := eastVal;
  Projected.Datum      := proj_tmp.Datum;
  Projected.ZoneNumber := intZoneNum;
End;



//////////////////////////////////////////////////////////////////////////
// meters (Northing/Easting) -> Latitude, Longitude
//////////////////////////////////////////////////////////////////////////
procedure TUTM.InverseTransform(Projected:TProjected; Var Coordinate:TCoordinate);
Var
  lat_origin, lon_origin    : Double;
  north_false, east_false   : Double;
  proj_tmp                  : TProjected;
  intZoneNum                : Integer;        // Zone: 1..60;
  east, north               : Double;
Begin
  // Input parameters;
  intZoneNum := Projected.ZoneNumber;
  east       := Projected.Easting;
  north      := Projected.Northing;

  //
  lat_origin  := 0.0;
  east_false  := 500000.0;

  // Set Zone parameters;
  lon_origin  := ((6 * abs(intZoneNum)) - 183);
  if (intZoneNum < 0) Then north_false := 10000000.0
  else north_false := 0.0;

  //
  north   := north - north_false;
  east    := east - east_false;

  proj_tmp.Northing   := north;
  proj_tmp.Easting    := east;
  proj_tmp.Datum      := Projected.Datum;
  Proj_tmp.ZoneNumber := intZoneNum;

  try
    InverseConversion(proj_tmp, lat_origin, lon_origin, scale_fact, Coordinate);
  except
     on EConversionInvalid do exit;
  End;
End;


//////////////////////////////////////////////////////////////////////////
// Function to calculate UTM zone number--NOTE Longitude entered in DEGREES!!!
//////////////////////////////////////////////////////////////////////////
function TUTM.calcZoneNum(Coordinate:TCoordinate):Integer;
Var
  zone                      : Integer;
  lat, lon                  : Double;
Begin
  lat := Coordinate.Latitude;
  lon := Coordinate.Longitude;

  zone := trunc((lon + 180)/6) + 1;

  if (lat >= 56.0) and (lat < 64.0) and (lon >= 3.0) and (lon < 12.0) Then zone := 32;

  // Special zones for Svalbard
  if (lat >= 72.0) and (lat < 84.0) Then
  Begin
    if (lon >= 0.0) and (lon <  9.0) Then zone := 31
    else if (lon >= 9.0) and (lon < 21.0) Then zone := 33
         else if (lon >= 21.0) and (lon < 33.0) Then zone := 35
	      else if (lon >= 33.0) and (lon < 42.0) Then zone := 37;
  End;
  result := zone;
End;


End.
